<?php
	/*
	 _____________
	 W I K I T E X T PHP-Class
	 
	 Version 2.0
	 
	 _____________
	 L I C E N S E
	 
	 Copyright (c) 2010, Alexander Sasse <alexander.sasse@xelaton.com>
	 
	 Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
	 
	 The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
	 
	 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
	 
	 _________
	 U S A G E
	 
	 $transform = new WikitextTransformer([Array of Strings, Filename, Text-String] $input);
	 echo $transform->performTransformation();
	 */
	
	class WikitextTransformer
	{
		var $TXTnl;
		var $transformNL;
		var $noWiki;
		var $mainLanguage;
		var $internalWikiLinks;
		
		var $search_array = array(
								  '/-{4,}/',
								  '/======(.*?)======/is',
								  '/=====(.*?)=====/is',
								  '/====(.*?)====/is',
								  '/===(.*?)===/is',
								  '/==(.*?)==/is',
								  '/\'\'\'\'\'(.*?)\'\'\'\'\'/is',
								  '/\'\'\'(.*?)\'\'\'/is',
								  '/\'\'(.*?)\'\'/is',
								  //'/(http:\/\/[a-zA-Z0-9-]?+\.?)(([a-zA-Z0-9-][a-zA-Z0-9-]+\.)+[a-zA-Z0-9-\.\/\_\?\%\#\&\=\;\~\!\(\)]+)/',
								  '/\[(?<!=")(http[s]{0,1}:\/\/(.*?)) (.*?)\]/',
								  '/\[(?<!=")(http[s]{0,1}:\/\/(.*?))\]/',
								  '/(?<!=")(http[s]{0,1}:\/\/(.*?)) /',
								  //'/\[\[(.*?):(.*?)\]\]/is',
								  //'/\[\[(.*?)\|(.*?)\]\]/is',
								  '/\[(.*?)\]/is',
								  '/\{\{lang\|(.*?)\|(.*?)\}\}/',
								  //		'/\{\{(.*?)\}\}/',
								  );
		//		{{lang|ja|二千年後の君へ ―シガンシナ陥落①―}}
		//		<span lang="ja" xml:lang="ja">微笑み ―ストヘス区急襲①―</span>
		
		var $replace_array;
		var $text_to_transform;
		
		function __construct($text_input) //WikitextTransformer($text_input)
		{
//            echo "Proboeren".$text_input;
			/*
			 if(is_array($text_input))
			 {
			 $this->text_to_transform = $text_input;
			 }
			 */
			if(file_exists($text_input))
			{
				//			$this->text_to_transform = explode("\n", file_get_contents($text_input));
				$this->text_to_transform = file_get_contents($text_input);
			}
			else
			{
				//			$this->text_to_transform = explode("\n", $text_input);
				$this->text_to_transform = $text_input;
			}
			
			$this->initVariables();
		}
		
		function initVariables()
		{
			$this->XTnl = "\n";
			$this->transformNL = "<br />";
			$this->noWiki = array();
			$this->langCode = "de";
			$this->internalWikiLinks = ".wikipedia.org/wiki/";
			$this->replace_array = array(
										 '<hr />',
										 '<h5>$1</h5>',
										 '<h4>$1</h4>',
										 '<h3>$1</h3>',
										 '<h2>$1</h2>',
										 '<h1>$1</h1>',
										 '<strong><em>$1</em></strong>',
										 '<strong>$1</strong>',
										 '<em>$1</em>',
										 '<a href="$1">$3</a>',
										 '<a href="$1">$1</a>',
										 '<a href="$1">$1</a>',
										 //'<a href="http://$1'.$this->internalWikiLinks.'$2">$2</a>',
										 //'<a href="http://'.$this->mainLanguage.$this->internalWikiLinks.'$1">$2</a>',
										 '<a href="$1">$1</a>',
										 '<span lang="$1" xml:lang="$1">$2</span>',
										 //'<span style="visibility:hidden;">$1</span>',
										 );
		}
		
		function performTransformation()
		{
			$this->saveNoWiki();
			
			$this->performInternalLinks();
			$this->simpleReplacing();
			$this->performDefinitionBlock();
			$this->performLists();
			$this->perfromReferences();
			$this->performTables();
			
			
			$this->infoboxFormatting();
			
			
			$this->makeParagraphs();
			$this->autoLineBreaks();
			$this->returnNoWiki();
			
			return $this->text_to_transform;
		}
		
		function simpleReplacing()
		{
			$this->text_to_transform = preg_replace($this->search_array, $this->replace_array, $this->text_to_transform);
		}
		
		function saveNoWiki()
		{
			$pattern = "/<nowiki>(.*?)<\/nowiki>/is";
//            echo $pattern."..".$this->text_to_transform;
			preg_match_all($pattern, $this->text_to_transform, $result);
			$counter = 1;
			for($i = 0; $i < count($result[0]); $i++)
			{
				$marker = "%%NO-WIKI-".$counter."%%";
				$this->noWiki[$marker] = $result[1][$i];
//                echo "<br />".$marker."--".$result[1][$i];
				$this->text_to_transform = str_replace($result[0][$i], $marker, $this->text_to_transform);
				$counter++;
			}
//            echo var_dump($this->noWiki);
		}
		
		function returnNoWiki()
		{
			foreach($this->noWiki as $key => $value)
			{
				$this->text_to_transform = str_replace($key, $value, $this->text_to_transform);
			}
		}
		
		function perfromReferences()
		{
			$pattern = "/<ref(.*?)>(.*?)<\/ref>/";
			preg_match_all($pattern, $this->text_to_transform, $result);
			//		echo "<pre>"; var_dump($result); echo "</pre>";
			if(count($result[0]) > 0)
			{
				$referenceList = "<ol>\n";
				$refCounter = 1;
				foreach($result[0] as $originNoteString)
				{
					$referenceDirty = substr($originNoteString, 5, strlen($originNoteString) - 11);
					$checkIndividualRefName = explode(">", $referenceDirty);
					$referenceValueClean = "";
					$referenceLinkClean = "";
					$referenceIDClean = "";
					if(count($checkIndividualRefName) > 1)
					{
						$referenceValueClean = $checkIndividualRefName[1];
						$referenceLinkDirty = $checkIndividualRefName[0];
						$referenceLinkDirty2 = str_replace(" ", "_", $referenceLinkDirty);
						
						$referenceLinkClean = substr($referenceLinkDirty2, 6, strlen($referenceLinkDirty2) - 7)."_ref";
						$referenceIDClean = substr($referenceLinkDirty2, 6, strlen($referenceLinkDirty2) - 7)."_note";
					}
					else
					{
						$referenceValueClean = $referenceDirty;
						$referenceLinkClean = "cite_ref-".$refCounter;
						$referenceIDClean = "cite_note-".$refCounter;
					}
					
					$replacementString = " <a href=\"".$referenceIDClean."\"><sup id=\"".$referenceLinkClean."\">[".$refCounter."]</sup></a>";
					
					$this->text_to_transform = str_replace($originNoteString, $replacementString, $this->text_to_transform);
					
					$referenceList .= "\t<li id=\"".$referenceIDClean."\"> <a href=\"#".$referenceLinkClean."\">&#x2191;</a> <small>".$referenceValueClean."</small></li>\n";
					$refCounter += 1;
				}
				$referenceList .= "</ol>";
				
				$this->text_to_transform = str_replace("<references/>", $referenceList, $this->text_to_transform);
				$this->text_to_transform = str_replace("<references />", $referenceList, $this->text_to_transform);
			}
			
		} // Ende perfromReferences() - Methode
		
		
		function performInternalLinks()
		{
			$pattern = "/(\[\[(.*?)\]\](.*?))[ \n]{1,1}/i";
			preg_match_all($pattern, $this->text_to_transform, $result);
			if(count($result[0]) > 0)
			{
				foreach($result[1] as $dirtyString)
				{
					$dirtyComps = explode("]]", $dirtyString);
					$addedLinkDisplayText = $dirtyComps[1];
					//				- anfang löschen
					$linkDefDirty = $dirtyComps[0];
					$linkDef = substr($linkDefDirty, 2);
					$replacementString;
					if(strpos($linkDef, "Wiktionary") !== false)
					{
						$wikiURLComps = explode($linkDef, "|");
						$wikiURLDirty = $wikiURLComps[0];
						$wikiURLDirty2 = substr($wikiURLDirty, 11);
						$wikiURL = str_replace(" ", "_", $wikiURLDirty2);
						
						$comps = explode(":", $linkDef);
						$displayDirty = array_pop($comps);
						$displayComps = explode("|", $displayDirty);
						
						if(count($displayComps) == 1)
						{
							$replacementString = "<a href=\"https://".$this->langCode.".wiktionary.org/wiki/".$wikiURL."\" alt=\"".$displayComps[0]."\" title=\"".$displayComps[ß]."\">".$linkDef."</a>";
						}
						else
						{
							//						echo "<pre>"; var_dump($displayComps); echo "</pre>";
							$displayItem = array_pop($displayComps);
							if(trim($displayItem) == "")
							{
								$finalDisplayString = substr($linkDef, 11, strlen($linkDef) - 12);
								$replacementString = "<a href=\"https://".$this->langCode.".wiktionary.org/wiki/".$wikiURL."\" alt=\"".$displayComps[0]."\" title=\"".$displayComps[0]."\">".$finalDisplayString."</a>";
							}
							else
							{
								$replacementString = "<a href=\"https://".$this->langCode.".wiktionary.org/wiki/".$wikiURL."\" alt=\"".$displayComps[0]."\" title=\"".$displayComps[0]."\">".$displayItem."</a>";
							}
						}
					}
					
					
					else if(strpos($linkDef, "Category") !== false)
					{
						$comps = explode(":", $linkDef);
						$firstItemDirty = $comps[0];
						if(trim($firstItemDirty) == "")
						{
							array_shift($comps);
							$wikiURLDirty =  implode(":", $comps);
							$wikiURLDirty2 = str_replace(" ", "_", $wikiURLDirty);
							$lastItem = array_pop($comps);
							$dirtyDisplayComps = explode("|", $lastItem);
							if(count($dirtyDisplayComps) > 1)
							{
								$wikiURL = substr($wikiURLDirty2, 0, strlen($wikiURLDirty2) - 1);
								$replacementString = "<a href=\"https://".$this->langCode.".wikipedia.org/wiki/".$wikiURL."\" alt=\"".$wikiURL."\" title=\"".$wikiURL."\">".$dirtyDisplayComps[0]."</a>";
							}
							else
							{
								$replacementString = "<a href=\"https://".$this->langCode.".wikipedia.org/wiki/".$wikiURLDirty2."\" alt=\"".$wikiURLDirty."\" title=\"".$wikiURLDirty."\">".$wikiURLDirty."</a>";
							}
						}
						else
						{
							// --> [[Category:Character sets]] --> wird normalerweise rausgelöscht und am Ende eines Artikels zu Kategorie-Auflistung dazugestellt, ich lasse es mal drinne, entferne aber die Klammern, damit es mit der LinkFormatierung nicht in die Quere kommt
							$replaceDirty1 = str_replace("[", " ", $dirtyString);
							$replacementString = str_replace("]", " ", $replaceDirty1);
						}
					}
					else if(strpos($linkDef, "File") !== false)
					{
						$fileDirty = substr($linkDef, 5, strlen($linkDef) - 6);
						$fileComps = explode("|", $fileDirty);
						$fileName = $fileComps[0];
						$replacementString = "<img src=\"".$fileName."\" />";
					}
					else
					{
						$checkAnchor = substr($linkDef, 0, 1);
						$comps = explode("|", $linkDef);
						$firstItemDirty = $comps[0];
						$firstItem = str_replace(" ", "_", $firstItemDirty);
						if(trim($checkAnchor) == "#")
						{
							if(count($comps) > 1)
							{
								$replacementString = "<a href=\"/".$firstItem."\" alt=\"".$comps[1]."\" title=\"".$comps[1]."\">".$comps[1].$addedLinkDisplayText."</a>";
							}
							else
							{
								$replacementString = "<a href=\"/".$firstItem."\" alt=\"".$comps[0]."\" title=\"".$comps[0]."\">".$comps[0].$addedLinkDisplayText."</a>";
							}
						}
						else
						{
							if(count($comps) > 1)
							{
								$replacementString = "<a href=\"https://".$this->langCode.".wikipedia.org/wiki/".$firstItem."\" alt=\"".$comps[1]."\" title=\"".$comps[1]."\">".$comps[0].$addedLinkDisplayText."</a>";
							}
							else
							{
								$replacementString = "<a href=\"https://".$this->langeCode.".wikipedia.org/wiki/".$firstItem."\" alt=\"".$comps[0]."\" title=\"".$comps[0]."\">".$comps[0].$addedLinkDisplayText."</a>";
							}
						}
					}
					$this->text_to_transform = str_replace($dirtyString, $replacementString, $this->text_to_transform, $replaceCount);
				} ## Ende Foreach Schleife
			}
		} // Ende performInternalLinks() - Methode
		
		function performLists()
		{
			$pattern = "/\n([^A-Za-z0-9]{0,})([*#])[ \t]{1,}(.*?)\n\n/is";
			preg_match_all($pattern, $this->text_to_transform, $result);
			//		echo "<pre>"; var_dump($result); echo "</pre>";
			
			foreach($result[0] as $singleList)
			{
				$formattedTable = "";
				$dirtyString = $singleList;
				$singleLines = explode("\n", $dirtyString);
				
				$listStarted = false;
				$currentLevel = 1;
				$openSubLists = 0;
				$closeListItems = array();
				$closeListWith = "";
				
				foreach($singleLines as $line)
				{
					$lineIsOrderedType = false;
					$dirtyLine = $line;
					if(strlen($dirtyLine) > 0)
					{
						$isListItem = false;
						
						$firstCharacter = $dirtyLine[0];
						if($firstCharacter == "#")
						{
							$lineIsOrderedType = true;
							$isListItem = true;
						}
						else if($firstCharacter == "*")
						{
							$isListItem = true;
						}
						else
						{
							// multiLine Zeile, zur vorherigen dazusetzen
							$isListItem = false;
							$formattedTable .= "<br />".dirtyLine;
						}
						
						if($isListItem)
						{
							if(!$listStarted)
							{
								if($lineIsOrderedType)
								{
									$formattedTable .= "\n<ol>";
									$closeListWith = "</ol>\n\n";
								}
								else
								{
									$formattedTable .= "\n<ul>";
									$closeListWith = "</ul>\n\n";
								}
								$position = strpos($dirtyLine, " ");
								$cleanString = substr($dirtyLine, $position);
								
								$formattedTable .= "\n\t<li>".$cleanString;
								$listStarted = true;
							}
							else
							{
								$position = strpos($dirtyLine, " ");
								
								$cleanString = substr($dirtyLine, $position);
								
								if($currentLevel == $position)
								{
									$addTabs = str_repeat("\t", $currentLevel);
									$formattedTable .= "</li>\n".$addTabs."<li>".$cleanString;
								}
								else if($currentLevel < $position) // level up, neue Liste eröffnen
								{
									$addTabs = str_repeat("\t", $currentLevel);
									$currentLevel += 1;
									$addTabs2 = str_repeat("\t", $currentLevel);
									if($lineIsOrderedType)
									{
										$formattedTable .= "\n".$addTabs."<ol>\n";
										$closeListItems[] = $addTabs."</ol></li>";
									}
									else
									{
										$formattedTable .= "\n".$addTabs."<ul>\n";
										$closeListItems[] = $addTabs."</ul></li>";
									}
									$formattedTable .=  $addTabs2."<li>".$cleanString;
									$openSubLists += 1;
								}
								else if($currentLevel > $position) // wir rücken runter, Listen schließen
								{
									// erst Listen schließen
									
									$counterForClosinglists = $position;
									while($counterForClosinglists < $currentLevel)
									{
										$counterForClosinglists += 1;
										$formattedTable .= "\n".array_pop($closeListItems)."\n";
									}
									
									// dann das nchste anhängen
									$currentLevel = $position;
									$addTabs = str_repeat("\t", $currentLevel);
									
									$formattedTable .= $addTabs."<li>".$cleanString;
								}
							}
						}
					}
				}
				
				$formattedTable .= "</li>\n";
				while(count($closeListItems) > 0)
				{
					$formattedTable .= array_pop($closeListItems)."\n";
				}
				$formattedTable .= $closeListWith;
				//			echo $formattedTable;
				$this->text_to_transform = str_replace($dirtyString, $formattedTable, $this->text_to_transform);
			}
			
		} // Ende performLists() - Methode
		
		function performTables()
		{
			$pattern = "/\{\|(.*?)\|\}\n/is";
			preg_match_all($pattern, $this->text_to_transform, $result);
			//		echo "<pre>"; var_dump($result); echo "</pre>";
			
			if(count($result[0]) > 0)
			{
				foreach($result[0] as $originNoteString)
				{
					$resultTable = "";
					$tablePieces = explode("\n", $originNoteString);
					array_pop($tablePieces); // letztes \nl entfernen
					array_pop($tablePieces); // Tabellenschluss entfernen |}
					
					// Anfang der Tabelle bearbeiten
					$tableStartDirty = array_shift($tablePieces);
					$tableStartcomps = explode("{|", $tableStartDirty);
					$addTableValueDirty = $tableStartcomps[1];
					$addTableValue = trim($addTableValueDirty);
					
					if(strlen($addTableValue) > 1)
					{
						$resultTable .= "\n<table ".$addTableValue.">\n";
					}
					else
					{
						$resultTable .= "\n<table>\n";
					}
					
					// erste Zeile der Tabelle bearbeiten, checken, ob caption vorhanden
					$checkCaption = array_shift($tablePieces);
					if(strpos($checkCaption, "|+") === 0)
					{
						$captionElements = explode("|+", $checkCaption);
						$captionValueDirty = $captionElements[1];
						$captionValueElements = explode("|", $captionValueDirty);
						if(count($captionValueElements)  > 1)
						{
							$captionStyleDirty = $captionValueElements[0];
							$captionDisplayDirty = $captionValueElements[1];
							$captionStyle = trim($captionStyleDirty);
							$captionDisplay = trim($captionDisplayDirty);
							$resultTable .= "\t<caption ".$captionStyle.">".$captionDisplay."</caption>\n\t<tbody>\n\t\t<tr>\n";
						}
						else
						{
							$cleanCaption = substr($checkCaption, 2);
							$resultTable .= "\t<caption>".$cleanCaption."</caption>\n\t<tbody>\n\t\t<tr>\n";
						}
					}
					else
					{
						$captionElements = explode("|-", $checkCaption);
						$captionValueDirty = $captionElements[1];
						$captionValue = trim($captionValueDirty);
						if($captionValue != "")
						{
							$resultTable .= "\t<tbody>\n\t\t<tr ".$captionValue.">\n";
						}
						else
						{
							$resultTable .= "\t<tbody>\n\t\t<tr>\n";
						}
					}
					
					// einzelne Zeilen durchgehen
					foreach($tablePieces as $line)
					{
						$trimmedStringDirty = $line;
						$trimmedString = trim($trimmedStringDirty);
						if(strpos($trimmedString, "|-") === 0)
						{
							$trElements = explode("|-", $trimmedString);
							$trValueDirty = $trElements[1];
							$trValue = trim($trValueDirty);
							if($trValue != "")
							{
								$resultTable .= "\t\t</tr>\n\t\t<tr ".$trValue.">\n";
							}
							else
							{
								$resultTable .= "\t\t</tr>\n\t\t<tr>\n";
							}
						}
						else
						{
							$indicator = $trimmedString[0];
							$cleanLine = substr($trimmedString, 1);
							$colFormatter = "td";
							$colSeperator = "||";
							if($indicator == "|")
							{
								$colFormatter = "td";
								$colSeperator = "||";
							}
							else // th's markiert durch ! !!
							{
								$colFormatter = "th";
								$colSeperator = "!!";
							}
							$colPieces = explode($colSeperator, $cleanLine);
							foreach($colPieces as $col)
							{
								$colString = $col;
								$colStringElements = explode("|", $colString);
								if(count($colStringElements) > 1)
								{
									$colElementStyleDirty = $colStringElements[0];
									$colElementStyle = trim($colElementStyleDirty);
									$colElementValueDirty = $colStringElements[1];
									$colElementValue = trim($colElementValueDirty);
									$resultTable .= "\t\t\t<".$colFormatter." ".$colElementStyle.">".$colElementValue."</".$colFormatter.">\n";
								}
								else
								{
									$colElementValueDirty = $col;
									$colElementValue = trim($colElementValueDirty);
									$resultTable .= "\t\t\t<".$colFormatter.">".$colElementValue."</".$colFormatter.">\n";
								}
							}
							//							}
						}
					}
					
					$resultTable .= "\t\t</tr>\n\t</tbody>\n</table>\n";
					$this->text_to_transform = str_replace($originNoteString, $resultTable, $this->text_to_transform);
				}
			}
			
			
		} // Ende performTables() - Methode
		
		function infoboxFormatting()
		{
			$result;
			$found_infoboxes = preg_match_all('/\{\{Infobox(.*?)\}\}/is', $this->text_to_transform, $result);
			$final_infobox_result;
			if($found_infoboxes >= 0)
			{
				foreach($result[0] as $single_infobox)
				{
					$single_infobox = substr_replace($single_infobox, "", strrpos($single_infobox, "}}"));
					$temp_rows = explode("|", $single_infobox);
					$infobox_result = "<table border=\"1\"><tr><th colspan=\"2\">".str_replace("{{Infobox", "", $temp_rows[0])."</th></tr>";
					for($i = 1; $i < count($temp_rows); $i++)
					{
						$temp_columns = explode(" = ", $temp_rows[$i]);
						$infobox_result .= "\n<tr>\n\t<td valign=\"top\">\n\t\t<strong>".$temp_columns[0]."</strong>\n\t</td>\n\t<td>".$temp_columns[1]."\n\t</td>\n</tr>";
					}
					$infobox_result .= "</table>";
					
					$this->text_to_transform = preg_replace('/\{\{Infobox(.*?)\}\}/is', $infobox_result, $this->text_to_transform , 1);
				}
			}
			return $this->text_to_transform;
		}
		
		function performDefinitionBlock()
		{
			$pattern = "/(?m)^[;](.*?)\n\n/is";
			preg_match_all($pattern, $this->text_to_transform, $result);
			//		echo "<pre>"; var_dump($result); echo "</pre>";
			//		echo "HAllo";
			if(count($result[0]) > 0)
			{
				$currentMatchIndex = 0;
				$finalString = "<dl>\n";
				while($currentMatchIndex < count($result[0]))
				{
					$dirtyList = $result[0][$currentMatchIndex];
					$dirtyPieces = explode(";", $dirtyList);
					$defTableCounter = 1; // erstes Element ist immer leer
					while($defTableCounter < count($dirtyPieces))
					{
						$termString = $dirtyPieces[$defTableCounter];
						$termArray = explode(":", $termString);
						$theTermDirty = $termArray[0];
						$theTerm = trim($theTermDirty);
						$finalString .= "\t<dt>".$theTerm."</dt>\n";
						$definitionCounter = 1;
						while($definitionCounter < count($termArray))
						{
							$theDefinitionDirty = $termArray[$definitionCounter];
							$theDefinitionDirty2 = trim($theDefinitionDirty);
							$theDefinition = str_replace("\n", "<br />", $theDefinitionDirty2);
							$finalString .= "\t<dd>".$theDefinition."</dd>\n";
							$definitionCounter += 1;
						}
						$defTableCounter += 1;
					}
					$finalString .= "</dl>\n\n";
					$this->text_to_transform = str_replace($dirtyList, $finalString, $this->text_to_transform);
					$currentMatchIndex += 1;
				}
			}
		}
		
		function makeParagraphs()
		{
			$addParasToText = explode("\n\n", $this->text_to_transform);
			for($i = 0; $i < count($addParasToText); $i++)
			{
				if(!is_numeric(strpos($addParasToText[$i], "<h")) &&
				   !is_numeric(strpos($addParasToText[$i], "<ol>")) &&
				   !is_numeric(strpos($addParasToText[$i], "</ol>")) &&
				   !is_numeric(strpos($addParasToText[$i], "<ul>")) &&
				   !is_numeric(strpos($addParasToText[$i], "</ul>")) &&
				   !is_numeric(strpos($addParasToText[$i], "<li>")) &&
				   !is_numeric(strpos($addParasToText[$i], "<p")) &&
				   !is_numeric(strpos($addParasToText[$i], "<table")) &&
				   !trim($addParasToText[$i]) == ""
				   )
				{
					$addParasToText[$i] = "<p>".$addParasToText[$i]."</p>";
				}
			}
			$this->text_to_transform = implode("\n\n", $addParasToText);
		} // Ende makeParagraphs() - Methode
		
		function autoLineBreaks()
		{
			$addBRToText = explode("\n", $this->text_to_transform);
			// helper variable to prevent <br /> in Tables
			$table_open = false;
			// going through all line
			for($i = 0; $i < count($addBRToText); $i++)
			{
				// check for LineEnding in this line
				if(is_numeric(strpos($addBRToText[$i], '<table')))
				{
					$table_open = true;
				}
				elseif(is_numeric(strpos($addBRToText[$i], "</table>")))
				{
					$table_open = false;
				}			
				// finding head and lines <h1> <hr> and list elements
				elseif(is_numeric(strpos($addBRToText[$i], "<h")) ||
					   is_numeric(strpos($addBRToText[$i], "<ol>")) ||
					   is_numeric(strpos($addBRToText[$i], "</ol>")) ||
					   is_numeric(strpos($addBRToText[$i], "<ul>")) ||
					   is_numeric(strpos($addBRToText[$i], "</ul>")) ||
					   is_numeric(strpos($addBRToText[$i], "<li>")) ||
					   is_numeric(strpos($addBRToText[$i], "</p>")) ||
					   is_numeric(strpos($addBRToText[$i], "<d")) ||
					   trim($addBRToText[$i]) == ""
					   )
				{
					// do nothing
				}
				elseif($table_open == false)
				{
					$addBRToText[$i] .= "<br />";
				}
			}
			$this->text_to_transform = implode("\n", $addBRToText);
		} // Ende autoLineBreaks() - Methode
	}
	?>
